import 'package:educational_audioplayer/loader.dart';
import 'package:educational_audioplayer/player.dart';
import 'package:flutter/material.dart';

const textRowHeight = 1.2;
const double secondaryHeaderFontSizeMultiplier = 0.85;

class AudioIcon extends StatelessWidget {
  final bool isSelected;
  AudioIcon({this.isSelected});

  @override
  Widget build(BuildContext context) {
    return Container(
      width: 48.0,
      height: 48.0,
      decoration: BoxDecoration(
          gradient: LinearGradient(
            // Where the linear gradient begins and ends
            begin: Alignment.topRight,
            end: Alignment.bottomLeft,
            // Add one stop for each color. Stops should increase from 0 to 1
            stops: [0.1, 0.5, 0.7, 0.9],
            colors: [
              // Colors are easy thanks to Flutter's Colors class.
              Colors.deepPurple[300],
              Colors.lightBlue[500],
              Colors.lightBlue[500],
              Colors.lightBlue[200],
            ],
          ),
          borderRadius: BorderRadius.all(Radius.circular(8.0))),
      child: Icon(Icons.volume_up,
          size: 30.0, color: isSelected ? Colors.white : Colors.white30),
    );
  }
}

class AudioItem extends StatelessWidget {
  final Player player;
  final List<Audio> audios;
  final int index;
  final double fontSize;
  final bool isSelected;
  final Function setLastAudioMethodLocal;

  AudioItem(
      {@required this.player,
      @required this.audios,
      @required this.index,
      @required this.fontSize,
      this.isSelected = false,
      this.setLastAudioMethodLocal});

  @override
  Widget build(BuildContext context) {
    return Container(
      color: isSelected ? Theme.of(context).highlightColor : null,
      child: ListTile(
          leading: AudioIcon(isSelected: isSelected),
          title: Text(audios[index].audioName,
              style: TextStyle(
                fontSize: fontSize,
              )),
          onTap: () {
            player.show();
            player.play(audios, index,
                setLastAudioMethodLocal: setLastAudioMethodLocal);
          }),
    );
  }
}

class LecturerItem extends StatelessWidget {
  final String title;
  final String subTitle;
  final double fontSize;
  final List<Audio> audios;

  LecturerItem(
      {@required this.title,
      @required this.subTitle,
      @required this.fontSize,
      @required this.audios});

  @override
  Widget build(BuildContext context) {
    return ListTile(
        title: Text(title,
            style: TextStyle(
                fontSize: fontSize,
                fontWeight: FontWeight.bold,
                height: textRowHeight)),
        subtitle: Text(subTitle,
            style: TextStyle(
                fontSize: fontSize * secondaryHeaderFontSizeMultiplier,
                color: Theme.of(context).unselectedWidgetColor,
                height: textRowHeight)),
        trailing: LoadDeleteButton(context: context, audios: audios));
  }
}
