import 'dart:async';
import 'dart:io';

import 'package:flutter/cupertino.dart';
import 'package:flutter/material.dart';
import 'package:flutter_plugin_playlist/flutter_plugin_playlist.dart';

import '../player.dart';
import '../util/loader.dart';

List<Audio> currentAudios = [Audio(url: '')];

class CommonPlayer extends StatefulWidget {
  final String imageUrl;
  CommonPlayer(
      {this.imageUrl:
          "https://ia801408.us.archive.org/34/items/speaker_202002/speaker.png"});

  @override
  CommonPlayerState createState() {
    return CommonPlayerState();
  }
}

class CommonPlayerState extends State<CommonPlayer> {
  RmxAudioPlayer player;
  int _audioPosition = 0;
  int _audioDuration = 0;
  int _audioIndex = 0;
  double _sliderValue = 0;
  double _sliderSeeking = 0;
  Function setLastAudioMethod;
  String previousAudioUrl = '';

  @override
  void initState() {
    super.initState();

    player = RmxAudioPlayer();
    player.initialize();
//    _setPlayList();
    player.on('status', (eventName, {dynamic args}) {
//      print(eventName + (args ?? "").toString());

      if ((args as OnStatusCallbackData).value != null) {
        setState(() {
//          currentIndex
          if ((args as OnStatusCallbackData).value['currentPosition'] != null) {
            _audioPosition =
                (args as OnStatusCallbackData).value['currentPosition'].toInt();
            _audioDuration =
                (((args as OnStatusCallbackData).value['duration']) ?? 0)
                    .toInt();
            _audioIndex =
                (args as OnStatusCallbackData).value['currentIndex'].toInt();

            if (_audioPosition > 0 && _audioDuration > 0) {
              _sliderValue = _audioPosition / _audioDuration;
            } else if (!player.isLoading && !player.isSeeking) {
              _sliderValue = 0;
            }

            if (_sliderSeeking != null &&
                !player.isSeeking &&
                !player.isLoading) {
              _sliderSeeking = null;
            }
          }
        });
      }
    });
  }

  @override
  void dispose() {
    player.clearAllItems();
    super.dispose();
  }

  String _formatTime(int secs) {
    int sec = secs;

    int min = 0;
    if (secs > 60) {
      min = (sec / 60).floor();
      sec = sec % 60;
    }

    return (min >= 10 ? min.toString() : '0' + min.toString()) +
        ":" +
        (sec >= 10 ? sec.toString() : '0' + sec.toString());
  }

  _updateLastAudio({Function setLastAudioMethodLocal}) async {
    String lastAudioUrl = currentAudios[_audioIndex].url;
    if (lastAudioUrl != previousAudioUrl) {
//      print(lastAudioUrl);
      if (setLastAudioMethodLocal is Function) {
//      print('setLastAudioMethodLocal is function');
        setLastAudioMethod = setLastAudioMethodLocal;
      } else if (setLastAudioMethod is Function) {
//      print('setLastAudioMethodLocal is not function');
//      print('setLastAudioMethod is function');
        setLastAudioMethod(lastAudioUrl);
        previousAudioUrl = lastAudioUrl;
      } else {
//      print('setLastAudioMethod is not function');
      }
    }
  }

  _setPlayList(List<Audio> list, {String image}) async {
    List<AudioTrack> playlist = [];

    List<String> audioUrls = [];

    for (int i = 0; i < list.length; i++) {
      String path = await getLocalPath(list[i].url);
      if ((await File(path).exists())) {
        String newPath = Platform.isIOS ? "file://" + path : path;
        audioUrls.add(newPath);
      } else {
        audioUrls.add(list[i].url);
      }
    }

    for (int i = 0; i < list.length; i++) {
      playlist.add(AudioTrack(
          albumArt: widget.imageUrl,
          trackId: list[i].url,
          album: list[i].chapterName,
          artist: list[i].authorName,
          assetUrl: audioUrls[i],
          title: list[i].audioName));
    }

    await player.setPlaylistItems(playlist,
        options: PlaylistItemOptions(startPaused: true));
  }

  Future startPlayList(List<Audio> audios, int index,
      {Function setLastAudioMethodLocal, String image}) async {
    currentAudios = audios;
    _updateLastAudio(setLastAudioMethodLocal: setLastAudioMethodLocal);
    await _setPlayList(audios, image: image);
    await player.playTrackByIndex(index);
  }

  play() async {
    setState(() {});
    await player.play();
  }

  Future pause() async {
    if (player.isLoading || player.isSeeking) return null;
    player.pause().then((_) {
//      print(_);
      setState(() {});
    }).catchError(print);
  }

  Future stop() async {
    player.clearAllItems();
  }

  Future resume() async {
    if (player.isLoading || player.isSeeking) return null;
    setState(() {});

    await play();
  }

  Future playNext() async {
    if (_audioIndex + 1 < currentAudios.length) {
      await player.skipForward();
    }
  }

  Future playPrevious() async {
    await player.skipBack();
  }

  addSecondsToPosition(double seconds) async {
    if (_audioDuration > 0) {
      double newPosition = _audioPosition + seconds;

      if (newPosition < 0) {
        newPosition = 0;
      }

      if (newPosition <= _audioDuration) {
        player.seekTo(newPosition);
      }
    }
  }

  double getSliderValue() {
    return _sliderValue;
  }

  double getSliderSeeking() {
    return _sliderSeeking;
  }

  String getSliderDurationText() {
    return _formatTime(_audioDuration);
  }

  String getSliderPositionText() {
    return _formatTime(_audioPosition);
  }

  onSliderChanged(double val) {
    if (_audioDuration > 0) {
      setState(() {
        _sliderSeeking = val;
      });
    }
  }

  onSliderChangeEnd(double val) async {
    if (_audioDuration > 0) {
      await player.seekTo(val * _audioDuration);
    }
  }

  onPlayPauseButtonPressed() {
    player.isPlaying ? pause() : player.isPaused ? resume() : play();
  }

  Widget getPlayPauseButton() {
    if (player.isLoading || player.isSeeking) {
      return const CircularProgressIndicator();
    }

    if (player.isPlaying) {
      return const Icon(Icons.pause);
    }

    return const Icon(Icons.play_arrow);
  }

  int getAudioIndex() {
    //    it is needed to update audio position of the parent widget
    _updateLastAudio();

    return _audioIndex;
  }

  @override
  Widget build(BuildContext context) {
    return Container();
  }
}
